//	Nandub - DivX/MPEG4v2 encoder
//	Copyright (C) 2001 Nando <nn@nandub.org>
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <windows.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#define __ECF_CPP__
#include "ecf.h"




char* strstrip( char* s, char* unwanted ) {
	
	char* p = s + strcspn( s, unwanted );
	while( *p ) {
		s = p;
		p = s + strspn( s, unwanted );
		strcpy( s, p );
		p = s + strcspn( s, unwanted );
	}

	return s;
}



// returns number of frames when successful otherwise <=0 
int new_frame_command ( char* in_cmd, int* out_first, framectl* xp )
{

	char* cmd = in_cmd;

	char* p = strchr( cmd, ':' );
	if( p==NULL ) return 0;

	(*p++) = 0;
	char* q = strchr( cmd, '-' );

	int first, last;
	if( q ) {
		(*q++) = 0;
		first = atoi( cmd );
		last  = atoi( q );
	} else {
		first = atoi( cmd );
		last  = first;
	}
	cmd = p;

	//
	// ok, we have the range, let's parse the commands
	//


	  // reset framectl
	  memset(xp,0,sizeof(framectl) );

	  while( cmd ) {
	  	  char *next = strchr( cmd, ',' );
		  if( next ) (*next++) = 0;
		  char* param = strchr( cmd, '=' );
		  if( param ) (*param++) = 0;

		  if( stricmp( cmd, "K" )==0 ) {
		  	  xp->flags |= FRAMECTL_FORCEKF;
			  xp->flags &= ~FRAMECTL_FORCEDF;
		  } else if( stricmp( cmd, "D" )==0 ) {
			  xp->flags |= FRAMECTL_FORCEDF;
			  xp->flags &= ~FRAMECTL_FORCEKF;
		  } else if( stricmp( cmd, "R" )==0 ) {
			  if( param ) {
				  xp->flags |= FRAMECTL_SETRATE;
				  xp->rate = atoi( param );
			  }
		  } else if( stricmp( cmd, "M" )==0 ) {
			  if( param ) {
				  xp->flags |= FRAMECTL_SETMOTION;
				  xp->motion = atoi( param );
			  }
		  } else if( stricmp( cmd, "CL" )==0 ) {
			  if( param ) {
				  xp->flags |= FRAMECTL_SETCL;
				  xp->cl = atoi( param );
			  }
		  } else if( stricmp( cmd, "G" )==0 ) {
			  if( param ) {
				  xp->flags |= FRAMECTL_SETGAUGE;
				  xp->gauge = atoi( param );
			  }
		  }

		  cmd = next;
	  }

	if( xp->flags==0 ) return 0; // doh! nothing to do

	*out_first = first;
	return last - first + 1;

}


static int compare_f( const void *arg1, const void *arg2 ) {
  return ((framectl *)arg1)->frameno - ((framectl *)arg2)->frameno;
}

int write_ecf ( char* fname )
{
	FILE* f = NULL;

	f = fopen ( fname,"wt");  
	if ( f == NULL ) return 0;
	
	int c = 0;
	DWORD flags;
	for ( int i = 0; i < ecfcount; i++ )
	{
		flags = ecftab[i].flags;

		fprintf(f,"@%i:",ecftab[i].frameno);
		if ( !flags ) {
			fprintf(f,"\n");
			continue;
		}

		if ( flags & FRAMECTL_FORCEKF ) {
			fprintf(f,"K");
			flags &= ~FRAMECTL_FORCEKF;
			if (flags ) fprintf(f,", ");
		}
		if ( flags & FRAMECTL_FORCEDF ) {
			fprintf(f,"D");
			flags &= ~FRAMECTL_FORCEDF;
			if (flags ) fprintf(f,", ");
		}

		if ( flags & FRAMECTL_SETRATE ) {
			fprintf(f,"R=%i",ecftab[i].rate);
			flags &= ~FRAMECTL_SETRATE;
			if (flags ) fprintf(f,", ");
		}

		if ( flags & FRAMECTL_SETCL )  {
			fprintf(f,"CL=%i",ecftab[i].cl);
			flags &= ~FRAMECTL_SETCL;
			if (flags ) fprintf(f,", ");
		}

		if ( flags & FRAMECTL_SETMOTION ){
			fprintf(f,"M=%i",ecftab[i].motion);
			flags &= ~FRAMECTL_SETMOTION;
			if (flags ) fprintf(f,", ");
		}

		if ( flags & FRAMECTL_SETGAUGE )  {
			fprintf(f,"G=%i",ecftab[i].gauge);
			flags |= ~FRAMECTL_SETGAUGE;
			if (flags ) fprintf(f,", ");
		}

		if ( flags & FRAMECTL_SETCRISPNESS ) fprintf(f,"C=%i",ecftab[i].crispness);

		fprintf(f,"\n");

	}

	fclose(f);
	return 1;
}

char line[4096]; // not on the stack please

int load_ecf( char* filename )
{

	// open ECF file
	FILE* f = fopen( filename, "r" );
	if( f==NULL ) {
		return -1;
	}
    

	ecfcount   = 0;
	framectl x = {0};
	int first  = 0;
	
	//
	// 1st pass - count frames
	//
	while( !feof( f ) ) {
		char* p;
		// read line 
		fgets( line, sizeof line, f );
		// replace  \n with 0
		p = strchr( line, '\n' ); if( p ) *p = 0;
		if( strlen( line )==0 ) continue;

		switch( line[0] ) {
		case '#': // comment
			continue;
			//break; /* not needed */
		case '@':
			strstrip( line+1, "\x09 \xA0\r\n" );
			// parse command
			ecfcount += new_frame_command( line+1, &first, &x );
			break;
		default:
			break;
		}
	}

	//
	// allocate memory for ecftab
	//
	ecftab = NULL;
	ecftab = (framectl*)malloc ( ecfcount * sizeof (framectl) );

	if ( ecftab == NULL ) return -1;
	
	// rewind the file
	fseek(f,0,SEEK_SET);

	//
	// 2nd pass - fill ecftab
    //

	int frameno = 0;
	int count;
	framectl* idx_ecftab =  ecftab;

	while( !feof( f ) ) {
		char* p;
		// read line 
		fgets( line, sizeof line, f );
		// replace  \n with 0
		p = strchr( line, '\n' ); if( p ) *p = 0;
		if( strlen( line )==0 ) continue;

		switch( line[0] ) {
		case '#': // comment
			continue;
			//break; /* not needed */
		case '@':
			strstrip( line+1, "\x09 \xA0\r\n" );
			// parse command
			count = new_frame_command( line+1, &frameno, &x );
			while (count > 0)
			{
				x.frameno = frameno;
				memcpy( idx_ecftab, &x, sizeof(framectl) );
				idx_ecftab++;
				count--;
				frameno++;
			}
			break;
		default:
			break;
		}
	}

    // sorting 
	if( ecfcount ) {
		qsort( ecftab, ecfcount, sizeof(framectl), compare_f );
	}

	fclose( f );

	//for DEBUG write_ecf("E:\\tmp\\test_ecf.txt");

	return ecfcount;
}


int compare_c( const void *arg1, const void *arg2 ) {
  return (*(int *)arg1) - ((framectl *)arg2)->frameno;
}

